package cz.muni.fi.pa165.common;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.transaction.annotation.Transactional;

public abstract class BaseService<T extends BaseObject> {

    public static final int DEFAULT_PAGE_SIZE = 10;

    public abstract JpaRepository<T, Long> getRepository();

    /**
     * Saves T typed entity into repository.
     *
     * @param entity entity to save.
     * @return saved entity.
     */
    @Transactional
    public T create(T entity) {
        return getRepository().save(entity);
    }

    /**
     * Retrieves all entities of type T with the given {@link Pageable} options in a read-only transaction.
     *
     * @param pageable the pagination information
     * @return a {@link Page} of entities of type T
     * @throws IllegalArgumentException in case the given {@link Pageable} is {@literal null}.
     */
    @Transactional(readOnly = true)
    public Page<T> findAll(Pageable pageable) {
        return getRepository().findAll(pageable);
    }

    /**
     *
     * Retrieves all entities of type T with the given page number and a default page size
     * in a read-only transaction.
     *
     * @param page the page number of the page to retrieve
     * @return a {@link Page} of entities of type T
     * @throws IllegalArgumentException if the given page number is less than zero.
     */
    @Transactional(readOnly = true)
    public Page<T> findAll(int page) {
        return getRepository().findAll(PageRequest.of(page, DEFAULT_PAGE_SIZE));
    }
}
